# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#           Luke Macken <lmacken@redhat.com>

import re

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url

from fedmsg_meta_fedora_infrastructure.conglomerators.bodhi import \
        requests as bodhi_requests
from fedmsg_meta_fedora_infrastructure.conglomerators.bodhi import \
        comments as bodhi_comments


def get_sync_product(msg):
    # Extract the product name from a ftpsync message.
    product = msg['topic'].split('.')[-2]
    if product == 'fedora':
        product = 'Fedora'
    elif product == 'epel':
        product = 'EPEL'
    else:
        pass
    return product


def is_ftp_sync(msg):
    return 'bodhi.updates.' in msg['topic'] and msg['topic'].endswith('.sync')


def author_link(username):
    return "<a href='https://admin.fedoraproject.org/updates/" + \
        "user/{user}'>{user}</a>".format(user=username)


def update_link(title):
    return "<a href='https://admin.fedoraproject.org/updates/" + \
        "{title}'>{title}</a>".format(title=title)


class BodhiProcessor(BaseProcessor):
    __name__ = "Bodhi"
    __description__ = "the Fedora update system"
    __link__ = "https://admin.fedoraproject.org/updates"
    __docs__ = "https://fedoraproject.org/wiki/Bodhi"
    __obj__ = "Package Updates"
    __icon__ = ("https://admin.fedoraproject.org/updates"
                "/static/images/bodhi-icon-48.png")
    conglomerators = [
        bodhi_requests.ByUserAndPackageTesting,
        bodhi_requests.ByUserAndPackageStable,
        bodhi_requests.ByPackage,
        bodhi_requests.ByUser,
        bodhi_comments.ByUpdate,
        bodhi_comments.ByUser,
    ]

    def _u2p(self, update):
        """ Take an update, and return the package name """
        # TODO -- make this unnecessary by having bodhi emit the
        # package name (and not just the update name) to begin with.
        return [build.rsplit('-', 2)[0] for build in update.split(',')]

    def title_or_alias(self, msg):
        value = msg['msg']['update'].get('alias')
        if not value:
            value = msg['msg']['update']['title']
        return value

    def secondary_icon(self, msg, **config):
        username = ''
        if 'bodhi.update.comment' in msg['topic']:
            username = msg['msg']['comment']['author']
        elif 'bodhi.buildroot_override' in msg['topic']:
            username = msg['msg']['override']['submitter']
        elif 'bodhi.stack' in msg['topic']:
            username = msg['msg']['agent']
        elif 'update' in msg['msg'] and 'submitter' in msg['msg']['update']:
            username = msg['msg']['update']['submitter']
        else:
            username = msg['msg'].get('agent')

        if isinstance(username, dict):
            username = username['name']

        avatar = self.__icon__
        if username:
            avatar = avatar_url(username)
        else:
            # If we don't have a user, then try with a package
            if 'update' in msg['msg']:
                packages = self._u2p(msg['msg']['update']['title'])
                tmpl = 'https://apps.fedoraproject.org/packages/' + \
                    'images/icons/%s.png'
                if packages:
                    avatar = tmpl % packages[0]

        return avatar

    def long_form(self, msg, **config):
        if 'bodhi.update.comment' in msg['topic']:
            return msg['msg']['comment']['text']
        elif 'bodhi.errata.publish' in msg['topic']:
            return msg['msg']['body']

    def subtitle(self, msg, **config):
        markup = config.get('markup', False)

        if is_ftp_sync(msg):
            product = get_sync_product(msg)
            msg = msg['msg']
            tmpl = self._(
                'New {product} {release} {repo} content synced out '
                '({bytes} changed with {deleted} files deleted)')
            return tmpl.format(product=product, **msg)
        elif 'bodhi.update.comment' in msg['topic']:
            author = msg['msg']['comment']['author']
            karma = msg['msg']['comment']['karma']
            title = msg['msg']['comment']['update_title']

            if len(title) >= 35:
                title = title[:35] + '...'

            tmpl = self._(
                "{author} commented on bodhi update {title} (karma: {karma})"
            )
            if not markup:
                return tmpl.format(author=author, karma=karma, title=title)
            else:
                return tmpl.format(
                    author=author_link(author),
                    title=update_link(title),
                    karma=karma,
                )

        elif 'bodhi.update.complete.' in msg['topic']:
            author = msg['msg']['update']['submitter']
            if isinstance(author, dict):
                author = author['name']
            package = msg['msg']['update']['title']
            status = msg['msg']['update']['status']
            tmpl = self._(
                "{author}'s {package} bodhi update completed push to {status}"
            )
            return tmpl.format(author=author, package=package, status=status)
        elif 'bodhi.update.eject' in msg['topic']:
            author = msg['msg']['update']['submitter']
            if isinstance(author, dict):
                author = author['name']
            package = msg['msg']['update']['title']
            repo = msg['msg']['repo']
            reason = msg['msg']['reason']
            tmpl = self._(
                "{author}'s {package} bodhi update was ejected from "
                "the {repo} mash.  Reason: \"{reason}\"")
            return tmpl.format(author=author, package=package,
                               repo=repo, reason=reason)
        elif 'bodhi.update.edit' in msg['topic']:
            author = msg['msg']['agent']
            update = msg['msg']['update']['title']
            tmpl = self._("{author} edited {update}")
            return tmpl.format(author=author, update=update)
        elif 'bodhi.update.request' in msg['topic']:
            status = msg['topic'].split('.')[-1]
            author = msg['msg'].get('agent')
            title = msg['msg']['update']['title']
            if status in ('unpush', 'obsolete', 'revoke'):
                # make our status past-tense
                status = status + (status[-1] == 'e' and 'd' or 'ed')
                tmpl = self._("{author} {status} {title}")
            else:
                tmpl = self._("{author} submitted {title} to {status}")

            if not markup:
                return tmpl.format(author=author, title=title, status=status)
            else:
                return tmpl.format(
                    author=author_link(author),
                    title=update_link(title),
                    status=status,
                )
        elif 'bodhi.mashtask.mashing' in msg['topic']:
            repo = msg['msg'].get('repo')
            tmpl = self._("bodhi masher started mashing {repo}")
            return tmpl.format(repo=repo)
        elif 'bodhi.mashtask.start' in msg['topic']:
            return self._("bodhi masher started a push")
        elif 'bodhi.mashtask.complete' in msg['topic']:
            success = msg['msg']['success']
            if success:
                tmpl = self._("bodhi masher successfully mashed {repo}")
            else:
                tmpl = self._("bodhi masher failed to mash {repo}")
            return tmpl.format(repo=msg['msg'].get('repo'))
        elif 'bodhi.mashtask.sync.wait' in msg['topic']:
            return self._("bodhi masher is waiting for {repo} to "
                          "hit the master mirror").format(
                              repo=msg['msg'].get('repo'))
        elif 'bodhi.mashtask.sync.done' in msg['topic']:
            return self._("bodhi masher finished waiting for {repo} to "
                          "hit the master mirror").format(
                              repo=msg['msg'].get('repo'))
        elif 'bodhi.buildroot_override.tag' in msg['topic']:
            tmpl = self._("{submitter} submitted a buildroot override " +
                          "for {build}")

            submitter = msg['msg']['override']['submitter']
            build = msg['msg']['override']['build']

            if isinstance(submitter, dict):
                submitter = submitter['name']
            if isinstance(build, dict):
                build = build['nvr']

            if markup:
                return tmpl.format(submitter=author_link(submitter),
                                   build=build)
            else:
                return tmpl.format(submitter=submitter, build=build)
        elif 'bodhi.buildroot_override.untag' in msg['topic']:
            tmpl = self._("{submitter} expired a buildroot override " +
                          "for {build}")

            submitter = msg['msg']['override']['submitter']
            build = msg['msg']['override']['build']

            if isinstance(submitter, dict):
                submitter = submitter['name']
            if isinstance(build, dict):
                build = build['nvr']

            if markup:
                return tmpl.format(submitter=author_link(submitter),
                                   build=build)
            else:
                return tmpl.format(submitter=submitter, build=build)
        elif 'bodhi.stack.save' in msg['topic']:
            tmpl = self._("{agent} updated the \"{name}\" stack")
            agent = msg['msg']['agent']
            name = msg['msg']['stack']['name']
            return tmpl.format(agent=agent, name=name)
        elif 'bodhi.stack.delete' in msg['topic']:
            tmpl = self._("{agent} deleted the \"{name}\" stack")
            agent = msg['msg']['agent']
            name = msg['msg']['stack']['name']
            return tmpl.format(agent=agent, name=name)
        elif 'bodhi.update.karma.threshold' in msg['topic']:
            tmpl = self._("{title} reached the {status} karma threshold")
            title = msg['msg']['update']['title']
            status = msg['msg']['status']
            return tmpl.format(title=title, status=status)
        elif 'bodhi.errata.publish' in msg['topic']:
            return msg['msg']['subject']

    def link(self, msg, **config):
        prefix = 'https://admin.fedoraproject.org'
        tmpl = prefix + "/updates/{title}"
        if 'bodhi.update.comment' in msg['topic']:
            return tmpl.format(title=msg['msg']['comment']['update_title'])
        elif 'bodhi.update.complete' in msg['topic']:
            return tmpl.format(title=self.title_or_alias(msg))
        elif 'bodhi.update.request' in msg['topic']:
            return tmpl.format(title=self.title_or_alias(msg))
        elif 'bodhi.update.edit' in msg['topic']:
            return tmpl.format(title=self.title_or_alias(msg))
        elif 'bodhi.update.eject' in msg['topic']:
            return tmpl.format(title=self.title_or_alias(msg))
        elif 'bodhi.update.karma.threshold' in msg['topic']:
            return tmpl.format(title=self.title_or_alias(msg))
        elif 'bodhi.errata.publish' in msg['topic']:
            return tmpl.format(title=self.title_or_alias(msg))
        elif 'bodhi.stack' in msg['topic']:
            return prefix + "/updates/stacks/{title}".format(
                title=msg['msg']['stack']['name'])
        elif is_ftp_sync(msg):
            link = "https://download.fedoraproject.org/pub/"
            repo = msg['msg']['repo']
            product = get_sync_product(msg).lower()
            link += product + "/"

            if product == 'fedora':
                if repo == 'atomic':
                    link += "linux/atomic/"
                else:
                    link += "linux/updates/"

            if repo.endswith('testing'):
                link += "testing/"

            link += msg['msg']['release'] + "/"
            return link

    def packages(self, msg, **config):
        if 'bodhi.update.comment' in msg['topic']:
            return set(self._u2p(msg['msg']['comment']['update_title']))
        elif 'bodhi.update.complete' in msg['topic']:
            return set(self._u2p(msg['msg']['update']['title']))
        elif 'bodhi.update.request' in msg['topic']:
            return set(self._u2p(msg['msg']['update']['title']))
        elif 'bodhi.update.edit' in msg['topic']:
            return set(self._u2p(msg['msg']['update']['title']))
        elif 'bodhi.update.eject' in msg['topic']:
            return set(self._u2p(msg['msg']['update']['title']))
        elif 'bodhi.update.karma.threshold' in msg['topic']:
            return set(self._u2p(msg['msg']['update']['title']))
        elif 'bodhi.errata.publish' in msg['topic']:
            return set(self._u2p(msg['msg']['update']['title']))
        elif 'bodhi.buildroot_override.' in msg['topic']:
            nvr = msg['msg']['override']['build']
            if isinstance(nvr, dict):
                nvr = nvr['nvr']
            return set(self._u2p(nvr))
        elif 'bodhi.stack' in msg['topic']:
            return set([p['name'] for p in msg['msg']['stack']['packages']])

        return set()

    def usernames(self, msg, **config):
        users = []

        for obj in ['update', 'override']:
            try:
                username = msg['msg'][obj]['submitter']
                if isinstance(username, dict):
                    username = username['name']
                users.append(username)
            except KeyError:
                pass

        try:
            users.append(msg['msg']['comment']['author'])
        except KeyError:
            pass

        if 'comment' in msg['msg']:
            text = msg['msg']['comment']['text']
            mentions = re.findall('@\w+', text)
            for mention in mentions:
                users.append(mention[1:])

        if 'agent' in msg['msg']:
            users.append(msg['msg']['agent'])

        # Strip out None values.  In some cases, we're getting None values
        # here.. I think from the 'agent' field of an anonymous bodhi comment.
        users = [u for u in users if u is not None]

        return set(users)

    def objects(self, msg, **config):
        if is_ftp_sync(msg):
            product = get_sync_product(msg).lower()
            msg = msg['msg']
            return set(['/'.join([product, msg['repo'], msg['release']])])
        elif 'bodhi.mashtask.mashing' in msg['topic']:
            return set(['repos/' + msg['msg'].get('repo')])
        elif 'bodhi.update.comment' in msg['topic']:
            return set([
                'packages/' + p for p in
                self._u2p(msg['msg']['comment']['update_title'])
            ])
        elif 'bodhi.update.complete' in msg['topic']:
            return set([
                'packages/' + p for p in
                self._u2p(msg['msg']['update']['title'])
            ])
        elif 'bodhi.update.request' in msg['topic']:
            return set([
                'packages/' + p for p in
                self._u2p(msg['msg']['update']['title'])
            ])
        elif 'bodhi.update.edit' in msg['topic']:
            return set([
                'packages/' + p for p in
                self._u2p(msg['msg']['update']['title'])
            ])
        elif 'bodhi.update.eject' in msg['topic']:
            return set([
                'packages/' + p for p in
                self._u2p(msg['msg']['update']['title'])
            ])
        elif 'bodhi.update.karma.threshold' in msg['topic']:
            return set([
                'packages/' + p for p in
                self._u2p(msg['msg']['update']['title'])
            ])
        elif 'bodhi.errata.publish' in msg['topic']:
            return set([
                'packages/' + p for p in
                self._u2p(msg['msg']['update']['title'])
            ])
        elif 'bodhi.buildroot_override.' in msg['topic']:
            nvr = msg['msg']['override']['build']
            if isinstance(nvr, dict):
                nvr = nvr['nvr']
            return set([
                'packages/' + p for p in
                self._u2p(nvr)
            ])
        elif 'bodhi.stack' in msg['topic']:
            return set(
                [
                    'packages/' + p['name']
                    for p in msg['msg']['stack']['packages']
                ] + ['stacks/' + msg['msg']['stack']['name']]
            )

        return set()



# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ricky Elrod <codeblock@fedoraproject.org>

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class JenkinsProcessor(BaseProcessor):
    __name__ = "Jenkins"
    __description__ = "Jenkins CI system"
    __link__ = "http://jenkins.cloud.fedoraproject.org/"
    __docs__ = "https://fedoraproject.org/wiki/Jenkins@infra"
    __obj__ = "Jenkins build status"
    __icon__ = ("https://wiki.jenkins-ci.org/download/attachments/"
                "2916393/logo.png?version=1")

    def subtitle(self, msg, **config):
        project = msg['msg'].get('project')

        if 'jenkins.build.failed' in msg['topic']:
            tmpl = self._("Jenkins project '{project}' failed to build")
        elif 'jenkins.build.passed' in msg['topic']:
            tmpl = self._("Jenkins project '{project}' built successfully")
        elif 'jenkins.build.unstable' in msg['topic']:
            tmpl = self._("Jenkins project '{project}' built with warnings")
        elif 'jenkins.build.notbuilt' in msg['topic']:
            tmpl = self._("Jenkins project '{project}' did not build")
        elif 'jenkins.build.aborted' in msg['topic']:
            tmpl = self._("Jenkins project '{project}' build aborted")
        elif 'jenkins.build.start' in msg['topic']:
            tmpl = self._("Jenkins project '{project}' started building")
        elif 'jenkins.build.unknown' in msg['topic']:
            tmpl = self._("Jenkins project '{project}' "
                          "entered an unknown state")
        else:
            raise NotImplementedError(msg['topic'])

        return tmpl.format(project=project)

    def link(self, msg, **config):
        build_id = msg['msg'].get('build')
        project = msg['msg'].get('project')

        if build_id is not None and project is not None:
            tmpl = ("http://jenkins.cloud.fedoraproject.org/job/{project}/"
                    "{build_id}/")
            return tmpl.format(build_id=build_id, project=project)
        else:
            return None

    def secondary_icon(self, msg, **config):
        return ''

    def usernames(self, msg, **config):
        return set()

    def objects(self, msg, **config):
        items = ['jenkins', msg['msg']['project']]
        items.append('.'.join(msg['topic'].split('.')[-2:]))
        return set(['/'.join(items)])



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Marek Brysa <mbrysa@redhat.com>
#
""" Tests for FAF (ABRT server) messages """

import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestFAFReport1(Base):
    """ `ABRT server <https://retrace.fedoraproject.org/>`_ notifies about a crash report
    """
    expected_title = "faf.report.threshold1"
    expected_subti = "ABRT report for package evolution has reached 7 occurrences"
    expected_packages = set(["evolution"])
    expected_usernames = set()
    expected_objects = set()
    expected_link = "http://example.org/faf/reports/1234/"
    expected_secondary_icon = "https://apps.fedoraproject.org/packages/images/icons/evolution.png"
    expected_long_form = """Packages: evolution
Function: main
First occurrence: 2015-04-10
Type:     core
Count:    7
URL:      http://example.org/faf/reports/1234/
"""
    msg = {
        u"username": u"faf",
        u"i": 1,
        u"timestamp": 1429777247,
        u"msg_id": u"2015-0284a675-995a-4942-bc59-2a5384e892b7",
        u"topic": u"org.fedoraproject.prod.faf.report.threshold1",
        u"msg": {
            u"count": 7,
            u"function": u"main",
            u"level": 1,
            u"url": u"http://example.org/faf/reports/1234/",
            u"components": [u"evolution"],
            u"type": u"core",
            u"first_occurrence": u"2015-04-10",
            u"report_id": 1234,
            u"problem_id": 4321
            }
        }


class TestFAFProblem10(Base):
    """ `ABRT server <https://retrace.fedoraproject.org/>`_ notifies about a problem
    """
    expected_title = "faf.problem.threshold10"
    expected_subti = "ABRT problem for package evolution, thunderbird has reached 77 occurrences"
    expected_packages = set(["evolution", "thunderbird"])
    expected_usernames = set()
    expected_objects = set()
    expected_link = "http://example.org/faf/problems/4321/"
    expected_secondary_icon = "https://apps.fedoraproject.org/packages/images/icons/evolution.png"
    expected_long_form = """Packages: evolution, thunderbird
Function: main
First occurrence: 2015-04-10
Type:     core
Count:    77
URL:      http://example.org/faf/problems/4321/
"""
    msg = {
        u"username": u"faf",
        u"i": 1,
        u"timestamp": 1429777247,
        u"msg_id": u"2015-0284a675-995a-4942-bc59-2a5384e892b7",
        u"topic": u"org.fedoraproject.prod.faf.problem.threshold10",
        u"msg": {
            u"count": 77,
            u"function": u"main",
            u"level": 1,
            u"url": u"http://example.org/faf/problems/4321/",
            u"components": [u"evolution", u"thunderbird"],
            u"type": u"core",
            u"first_occurrence": u"2015-04-10",
            u"problem_id": 4321
            }
        }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class KarmaProcessor(BaseProcessor):
    __name__ = "irc"
    __description__ = "IRC Karma"
    __link__ = "https://fedoraproject.org/wiki/Zodbot"
    __docs__ = "https://fedoraproject.org/wiki/Zodbot"
    __obj__ = "IRC Karma"
    __icon__ = "https://apps.fedoraproject.org/img/icons/meetbot.png"

    def subtitle(self, msg, **config):
        if msg['msg']['vote'] == 1:
            tmpl = self._('{agent} gave {recipient}({total}) '
                          'a karma cookie in {channel}')
        else:
            tmpl = self._('{agent} gave {recipient}({total}) '
                          'negative karma in {channel}')

        # Backwards compat
        if 'line' in msg['msg']:
            tmpl = tmpl + self._('.  "{line}"')

        return tmpl.format(**msg['msg'])

    def link(self, msg, **config):
        user = msg['msg']['recipient']
        return 'https://badges.fedoraproject.org/user/' + user

    def icon(self, msg, **config):
        return avatar_url(msg['msg']['agent'])

    def secondary_icon(self, msg, **config):
        return avatar_url(msg['msg']['recipient'])

    def usernames(self, msg, **config):
        return set([msg['msg']['agent'], msg['msg']['recipient']])

    def objects(self, msg, **config):
        return set(['karma/' + msg['msg']['recipient']])



# This file is part of fedmsg
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for mailman3 messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestMailman3NewReply(Base):
    """ `Discussion lists for the Fedora Project
    <https://lists.fedoraproject.org>`_ run on mailman3.  When a new
    message is published on a list, fedmsg will pop out one of these messages.
    The following is an example of a reply to a thread.
    """

    expected_title = "mailman.receive"
    expected_subti = ("On the devel list, nim replied to "
                      "'[Devel] Re:Software Management call for RFEs'")
    expected_link = ("https://lists.fedoraproject.org/archives/list/"
                     "devel@mm3test.fedoraproject.org/message/"
                     "S3PHLMD7PGWXXLBN3GENHVK7JJ37UWLJ/")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "e13c266c61de5e79cf37a99b94d4d4462d6857fe8a5dbdde9e041ae01b14a7db"
        "?s=64&d=retro")

    expected_packages = set([])
    expected_usernames = set([
        # We convert emails to fas usernames if we can.
        'nim',
    ])
    expected_objects = set([
        # There is only one item in this set, not four.  It's just long.
        "5de4f14ae46cce6de03cf68ca06526a9.squirrel@arekh.dyndns.org/"
        "519DFB93.1060502@laiskiainen.org/"
        "d4f0cefb4a7b845451ecab2c4026fe4d.squirrel@arekh.dyndns.org/"
        "message",
    ])
    msg = {
        "i": 4,
        "msg": {
            "mlist": {
                "list_name": "devel"
            },
            "msg": {
                "delivered-to": "devel@lists.fedoraproject.org",
                "from": "\"Nicolas Mailhot\" <nicolas.mailhot@laposte.net>",
                "x-mailman-rule-hits": "nonmember-moderation",
                "to": "\"Development discussions related to Fedora\" "
                "<devel@lists.fedoraproject.org>",
                "cc": None,
                "in-reply-to": "<519DFB93.1060502@laiskiainen.org>",
                "x-message-id-hash": "S3PHLMD7PGWXXLBN3GENHVK7JJ37UWLJ",
                "x-mailman-rule-misses": "approved; emergency; loop; "
                "member-moderation; administrivia; implicit-dest; "
                "max-recipients; max-size; news-moderation; no-subject; "
                "suspicious-header",
                "references": "<5de4f14ae46cce6de03cf68ca06526a9.squirrel@"
                "arekh.dyndns.org>\n\t<519DFB93.1060502@laiskiainen.org>",
                "archived-at": "https://lists.fedoraproject.org/archives/"
                "list/devel@mm3test.fedoraproject.org/message/"
                "S3PHLMD7PGWXXLBN3GENHVK7JJ37UWLJ/",
                "message-id": "<d4f0cefb4a7b845451ecab2c4026fe4d.squirrel@"
                "arekh.dyndns.org>",
                "subject": "[Devel] Re:Software Management call for RFEs"
            }
        },
        "topic": "org.fedoraproject.dev.mailman.receive",
        "username": "mailman",
        "timestamp": 1369322289.6794021
    }


class TestMailman3NewMail(Base):
    """ `Discussion lists for the Fedora Project
    <https://lists.fedoraproject.org>`_ run on mailman3.  When a new
    message is published on a list, fedmsg will pop out one of these messages.
    The following is an example of a new thread being started.
    """

    expected_title = "mailman.receive"
    expected_subti = ("jreznik@redhat.com wrote '[Devel] Fedora 19 Beta status"
                      " is Go, release on May 28, 2013' to the devel list")
    expected_link = ("https://lists.fedoraproject.org/archives/list/"
                     "devel@mm3test.fedoraproject.org/message/"
                     "HDMTECNRNUHZTSDGM2FDK6LGCMAS2PZ4/")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "35012533ff5290bd2231c7133bd07896?s=64&d=retro")

    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        "306436886.6773069.1369333725371.JavaMail.root@redhat.com/message",
    ])
    msg = {
        "i": 1,
        "msg": {
            "mlist": {
                "list_name": "devel"
            },
            "msg": {
                "delivered-to": "devel@lists.fedoraproject.org",
                "from": "Jaroslav Reznik <jreznik@redhat.com>",
                "x-mailman-rule-hits": "nonmember-moderation",
                "to": "devel-announce@lists.fedoraproject.org,\n\t"
                "test-announce@lists.fedoraproject.org,\n\t"
                "Fedora Logistics List <logistics@lists.fedoraproject.org>",
                "cc": None,
                "in-reply-to": None,
                "x-message-id-hash": "HDMTECNRNUHZTSDGM2FDK6LGCMAS2PZ4",
                "x-mailman-rule-misses": "approved; emergency; loop; "
                "member-moderation; administrivia; implicit-dest; "
                "max-recipients; max-size; news-moderation; no-subject; "
                "suspicious-header",
                "references": None,
                "archived-at": "/list/devel@mm3test.fedoraproject.org/"
                "message/HDMTECNRNUHZTSDGM2FDK6LGCMAS2PZ4/",
                "message-id": "<306436886.6773069.1369333725371.JavaMail."
                "root@redhat.com>",
                "subject": "[Devel] Fedora 19 Beta status is Go, release on "
                "May 28, 2013"
            }
        },
        "topic": "org.fedoraproject.dev.mailman.receive",
        "username": "mailman",
        "timestamp": 1369334087.9298041
    }


add_doc(locals())



# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>

import requests

from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url
from fedmsg_meta_fedora_infrastructure import BaseProcessor


class GithubProcessor(BaseProcessor):
    __name__ = "github"
    __description__ = "Fedora-related Github Repos"
    __link__ = "https://github.com"
    __icon__ = "https://apps.fedoraproject.org/img/icons/github.png"
    __docs__ = "https://developer.github.com/webhooks/#events"
    __obj__ = "Github Events"

    def _get_user(self, msg):
        if msg['msg'].get('commit', None):
            user = msg['msg']['commit'].get('author')
            if user:
                user = user.get('login')
            if user:
                return msg['msg']['fas_usernames'].get(user, user)
        if msg['msg'].get('pusher', None):
            pusher = msg['msg']['pusher']['name']
            return msg['msg']['fas_usernames'].get(pusher, pusher)
        if msg['msg'].get('sender', None):
            sender = msg['msg']['sender']['login']
            return msg['msg']['fas_usernames'].get(sender, sender)
        if msg['msg'].get('forkee', None):
            forkee = msg['msg']['forkee']['owner']['login']
            return msg['msg']['fas_usernames'].get(forkee, forkee)
        return None

    def _get_repo(self, msg):
        try:
            repo = msg['msg']['repository']
            if 'login' in repo['owner']:
                return repo['owner']['login'] + '/' + repo['name']
            else:
                return repo['owner']['name'] + '/' + repo['name']

        except KeyError:
            return None

    def link(self, msg, **config):
        if 'github.watch' in msg['topic']:
            return msg['msg']['repository']['html_url'] + "/watchers"
        if 'github.star' in msg['topic']:
            return msg['msg']['repository']['html_url'] + "/stargazers"
        if 'github.page_build' in msg['topic']:
            temp = msg['msg']['repository']['html_url'].split("/")
            return 'http://{0}.github.io/{1}/'.format(temp[-2],temp[-1])
        if 'release' in msg['msg']:
            return msg['msg']['release']['html_url']
        if 'target_url' in msg['msg']:
            return msg['msg']['target_url']
        if 'compare' in msg['msg']:
            return msg['msg']['compare']
        if 'comment' in msg['msg']:
            return msg['msg']['comment']['html_url']
        if 'pull_request' in msg['msg']:
            return msg['msg']['pull_request']['html_url']
        if 'issue' in msg['msg']:
            return msg['msg']['issue']['html_url']
        if 'forkee' in msg['msg']:
            return msg['msg']['forkee']['html_url']
        if 'repository' in msg['msg']:
            return msg['msg']['repository']['html_url']

    def long_form(self, msg, **config):
        if '.pull_request_review_comment' in msg['topic']:
            body = msg['msg']['comment']['body']
            return body
        if '.issue.comment' in msg['topic']:
            body = msg['msg']['comment']['body']
            return body
        if '.issue.reopened' in msg['topic']:
            body = msg['msg']['issue']['body']
            return body
        if 'commit_comment' in msg['topic']:
            body = msg['msg']['comment']['body']
            return body
        elif 'github.push' in msg['topic']:
            url = msg['msg']['compare'] + ".patch"
            response = requests.get(url)
            if response.status_code == 200:
                return response.text

    def subtitle(self, msg, **config):
        user = self._get_user(msg)
        repo = self._get_repo(msg)
        if 'github.webhook' in msg['topic']:
            return self._('A new github repository has been added to fedmsg')
        elif 'github.push' in msg['topic']:
            n = len(msg['msg']['commits'])
            tmpl = self._('{user} pushed {n} commit(s) to {repo}')
            return tmpl.format(user=user, repo=repo, n=n)
        elif 'github.pull_request.' in msg['topic']:
            action = msg['msg']['action']
            n = msg['msg']['number']
            tmpl = self._('{user} {action} pull request #{n} on {repo}')
            return tmpl.format(user=user, action=action, n=n, repo=repo)
        elif 'github.issue.' in msg['topic']:
            if 'comment' in msg['msg']:
                action = 'commented on'
            else:
                action = msg['msg']['action']

            try:
                n = msg['msg']['number']
            except KeyError:
                n = msg['msg']['issue']['number']

            tmpl = self._('{user} {action} issue #{n} on {repo}')
            return tmpl.format(user=user, action=action, n=n, repo=repo)
        elif 'github.pull_request_review_comment' in msg['topic']:
            n = msg['msg']['pull_request']['number']
            tmpl = self._('{user} commented on PR #{n} on {repo}')
            return tmpl.format(user=user, n=n, repo=repo)
        elif 'github.commit_comment' in msg['topic']:
            tmpl = self._('{user} commented on a commit on {repo}')
            return tmpl.format(user=user, repo=repo)
        elif 'github.create' in msg['topic']:
            typ = msg['msg']['ref_type']
            ref = msg['msg']['ref']
            tmpl = self._('{user} created a new {typ} "{ref}" at {repo}')
            return tmpl.format(user=user, repo=repo, ref=ref, typ=typ)
        elif 'github.delete' in msg['topic']:
            typ = msg['msg']['ref_type']
            ref = msg['msg']['ref']
            tmpl = self._('{user} deleted the "{ref}" {typ} at {repo}')
            return tmpl.format(user=user, repo=repo, ref=ref, typ=typ)
        elif 'github.fork' in msg['topic']:
            tmpl = self._('{user} forked {repo}')
            return tmpl.format(user=user, repo=repo)
        elif 'github.status' in msg['topic']:
            description = msg['msg']['description']
            sha = msg['msg']['sha'][:8]
            tmpl = self._("{description} for {repo} {sha}")
            return tmpl.format(description=description, repo=repo, sha=sha)
        elif 'github.watch' in msg['topic']:
            tmpl = self._('{user} {action} watching {repo}')
            action = msg['msg']['action']
            return tmpl.format(user=user, repo=repo, action=action)
        elif 'github.star' in msg['topic']:
            tmpl = self._('{user} {action} {repo}')
            action = msg['msg']['action']
            if action == 'started':
                action = 'starred'
            else:
                action = 'unstarred'
            return tmpl.format(user=user, repo=repo, action=action)
        elif 'github.release' in msg['topic']:
            tmpl = self._('{user} cut a release of {repo}, version {version}')
            version = msg['msg']['release']['tag_name']
            repo = repo.split('/', 1)[1]
            return tmpl.format(user=user, repo=repo, version=version)
        elif 'github.page_build' in msg['topic']:
            tmpl = self._('{user} rebuilt the github.io page for {repo}')
            return tmpl.format(user=user, repo=repo)
        elif 'github.team_add' in msg['topic']:
            tmpl = self._("The '{team_name}' team was added to the {repo} repository")
            teamName = msg['msg']['team']['slug']
            return tmpl.format(team_name=teamName,repo=repo)
        elif 'github.member' in msg['topic']:
            if msg['msg']['action'] == 'added':
                tmpl = self._("{user} added {member} as a member of {repo}")
            else:
                tmpl = self._("{user} removed member {member} from {repo}")
            member = msg['msg']['member']['login']
            return tmpl.format(user=user,member=member,repo=repo)
        else:
            pass

    def secondary_icon(self, msg, **config):
        if 'github.webhook' in msg['topic']:
            return self.icon(msg, **config)
        # Otherwise
        return avatar_url(self._get_user(msg))

    def usernames(self, msg, **config):
        return set([name for name in msg['msg']['fas_usernames'].values()
                    if not name.startswith('github_org_')])

    def objects(self, msg, **config):
        suffix = '.'.join(msg['topic'].split('.')[3:5])
        lookup = {
            'github.push': 'tree',
            'github.issue': 'issues',
            'github.fork': 'forks',
            'github.status': 'status',
            'github.pull_request': 'pull',
            'github.pull_request_review_comment': 'pull',
            'github.commit_comment': 'tree',
            'github.release': 'releases',
            'github.create': None,
            'github.delete': None,
            'github.watch': None,
            'github.star': None,
            'github.page_build': 'page_build',
            'github.team_add': 'team_add',
            'github.member': 'member'
        }

        if suffix not in lookup:
            return set()

        base = self._get_repo(msg)
        if lookup[suffix]:
            base = base + "/" + lookup[suffix]

        items = []
        if 'commits' in msg['msg']:
            for c in msg['msg']['commits']:
                items = c['added'] + c['removed'] + c['modified']
        elif suffix == 'github.pull_request':
            items = ['%s' % msg['msg']['number']]
        elif suffix == 'github.issue':
            try:
                n = msg['msg']['number']
            except KeyError:
                n = msg['msg']['issue']['number']
            items = ['%s' % n]
        elif suffix == 'github.pull_request_review_comment':
            n = msg['msg']['pull_request']['number']
            items = ['%s' % n]
        elif suffix == 'github.fork':
            items = [self._get_user(msg)]
        elif suffix == 'github.create' or suffix == 'github.delete':
            items = ['/'.join([msg['msg']['ref_type'], msg['msg']['ref']])]
        elif suffix == 'github.status':
            items = [msg['msg']['commit']['sha']]
        elif suffix == 'github.commit_comment':
            items = [msg['msg']['comment']['path']]
        elif suffix == 'github.release':
            items = [msg['msg']['release']['tag_name']]
        elif suffix == 'github.watch':
            items = ['watchers']
        elif suffix == 'github.star':
            items = ['stargazers']
        if not items:
            return set([base])
        return set([base + '/' + item for item in items])



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Pierre-Yves Chibon <pingou@pingoured.fr>
#

from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url, avatar_url_from_email, email2fas
from fedmsg_meta_fedora_infrastructure import BaseProcessor
import fedmsg.meta.base

class PagureProcessor(BaseProcessor):
    topic_prefix_re = 'io\\.pagure\\.(dev|stg|prod)'

    __name__ = "pagure"
    __description__ = "Pagure forge"
    __link__ = "https://pagure.io"
    __docs__ = "https://pagure.io/pagure"
    __obj__ = "Pagure forge"
    __icon__ = ("https://apps.fedoraproject.org/packages/"
                "images/icons/package_128x128.png")

    def __get_project(self, msg):
        ''' Return the project as `foo` or `user/foo` if the project is a
        fork.
        '''
        project = msg['project']['name']
        if msg['project']['parent']:
            user = msg['project']['user']['name']
            project = '/'.join([user, project])
        return project

    def link(self, msg, **config):
        try:
            project = self.__get_project(msg['msg'])
        except KeyError:
            project = self.__get_project(msg['msg']['pullrequest'])
        base_url = "https://pagure.io"

        tmpl = '{base_url}/{project}'
        if '/' in project:
            tmpl = '{base_url}/fork/{project}'
        if 'pagure.issue' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            tmpl += '/issue/{id}'
            return tmpl.format(
                base_url=base_url, project=project, id=issueid)
        elif 'pagure.project' in msg['topic']:
            return tmpl.format(base_url=base_url, project=project)
        elif 'pagure.pull-request' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            tmpl += '/pull-request/{id}'
            return tmpl.format(
                base_url=base_url, project=project, id=prid)
        else:
            return base_url

        return None

    def subtitle(self, msg, **config):
        try:
            project = self.__get_project(msg['msg'])
        except KeyError:
            project = self.__get_project(msg['msg']['pullrequest'])
        user = msg['msg']['agent']

        if 'pagure.project.new' in msg['topic']:
            tmpl = self._(
                '{user} created a new project "{project}"'
            )
            return tmpl.format(user=user, project=project)
        elif 'pagure.issue.new' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            title = msg['msg']['issue']['title']
            tmpl = self._(
                '{user} opened a new ticket {project}#{id}: "{title}"'
            )
            return tmpl.format(
                user=user, project=project, title=title, id=issueid)
        elif 'pagure.issue.comment.added' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            title = msg['msg']['issue']['title']
            tmpl = self._(
                '{user} commented on the ticket {project}#{id}: "{title}"'
            )
            return tmpl.format(
                user=user, project=project, title=title, id=issueid)
        elif 'pagure.issue.tag.added' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            tags = ', '.join(msg['msg']['tags'])
            tmpl = self._(
                '{user} tagged the ticket {project}#{id}: {tags}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid, tags=tags)
        elif 'pagure.issue.tag.removed' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            tags = ', '.join(msg['msg']['tags'])
            tmpl = self._(
                '{user} removed tags: {tags}, from the ticket {project}#{id}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid, tags=tags)
        elif 'pagure.issue.assigned.added' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            assignee = msg['msg']['issue']['assignee']['name']
            tmpl = self._(
                '{user} assigned {assignee} to the ticket {project}#{id}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid, assignee=assignee)
        elif 'pagure.issue.assigned.reset' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            tmpl = self._(
                '{user} reset the assignee of the ticket {project}#{id}'
            )
            return tmpl.format(user=user, project=project, id=issueid)
        elif 'pagure.issue.dependency.added' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            dep_id = msg['msg']['added_dependency']
            tmpl = self._(
                '{user} added on the ticket {project}#{id} a dependency on '
                'ticket {project}#{dep_id}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid,
                blocked=', '.join(blocked))
        elif 'pagure.issue.dependency.removed' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            removed = msg['msg']['removed_dependency']
            tmpl = self._(
                '{user} removed on the ticket {project}#{id} the dependency '
                'on ticket {project}#{removed}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid, removed=removed)
        elif 'pagure.issue.edit' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            fields = '", "'.join(msg['msg']['fields'])
            tmpl = self._(
                '{user} edited the fields: "{fields}" of the ticket '
                '{project}#{id}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid, fields=fields)
        elif 'pagure.project.edit' in msg['topic']:
            fields = '", "'.join(msg['msg']['fields'])
            tmpl = self._(
                '{user} edited the fields: "{fields}" of the project '
                '{project}'
            )
            return tmpl.format(user=user, project=project, fields=fields)
        elif 'pagure.project.user.added' in msg['topic']:
            new_user = msg['msg']['new_user']
            tmpl = self._(
                '{user} added "{new_user}" to the project {project}'
            )
            return tmpl.format(user=user, project=project, new_user=new_user)
        elif 'pagure.project.tag.removed' in msg['topic']:
            tags = '", "'.join(msg['msg']['tags'])
            tmpl = self._(
                '{user} removed tags "{tags}" of the project {project}'
            )
            return tmpl.format(user=user, project=project, tags=tags)
        elif 'pagure.project.tag.edited' in msg['topic']:
            old_tag = msg['msg']['old_tag']
            new_tag = msg['msg']['new_tag']
            tmpl = self._(
                '{user} edited tags "{old_tag}" of the project {project} '
                'to "{new_tag}"'
            )
            return tmpl.format(
                user=user, project=project,
                old_tag=old_tag, new_tag=new_tag)
        elif 'pagure.project.forked' in msg['topic']:
            old_project = msg['msg']['project']['parent']['name']
            tmpl = self._(
                '{user} forked project "{old_project}" to "{project}"'
            )
            return tmpl.format(
                user=user, old_project=old_project, project=project)
        elif 'pagure.pull-request.comment.added' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            tmpl = self._(
                '{user} commented on the pull-request#{id} of project '
                '"{project}"'
            )
            return tmpl.format(user=user, id=prid, project=project)
        elif 'pagure.pull-request.closed' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            merged = msg['msg']['merged']
            if merged:
                tmpl = self._(
                    '{user} merged the pull-request#{id} of project '
                    '"{project}"'
                )
            else:
                tmpl = self._(
                    '{user} closed (without merging) the pull-request#{id} '
                    'of project "{project}"'
                )
            return tmpl.format(user=user, id=prid, project=project)
        elif 'pagure.pull-request.new' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            title = msg['msg']['pullrequest']['title']
            tmpl = self._(
                '{user} opened the pull-request#{id}: "{title}" for the '
                'project "{project}"'
            )
            return tmpl.format(
                user=user, id=prid, project=project, title=title)
        elif 'pagure.pull-request.flag.added' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            username = msg['msg']['flag']['username']
            comment = msg['msg']['flag']['comment']
            tmpl = self._(
                '{username} flagged {project} #{id} with "{comment}"'
            )
            return tmpl.format(username=username, id=prid, comment=comment)

        elif 'pagure.pull-request.flag.updated' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            username = msg['msg']['flag']['username']
            comment = msg['msg']['flag']['comment']
            tmpl = self._(
                '{username} updated the flag of {project} #{id} with: '
                '"{comment}"'
            )
            return tmpl.format(username=username, id=prid, comment=comment)

        else:
            pass

    def secondary_icon(self, msg, **config):
        username = msg['msg'].get('agent')
        if username:
            return avatar_url(username)
        else:
            return None

    def usernames(self, msg, **config):
        username = msg['msg'].get('agent')
        if username:
            return set([username])
        else:
            return set([])

    def objects(self, msg, **config):
        try:
            project = self.__get_project(msg['msg'])
        except KeyError:
            project = self.__get_project(msg['msg']['pullrequest'])

        if 'pagure.project' in msg['topic']:
            return set([
                'project/%s' % project,
            ])
        elif 'pagure.issue' in msg['topic']:
            return set([
                'issue/%s' % msg['msg']['issue']['id'],
                'project/%s' % project,
            ])
        elif 'pagure.pull-request' in msg['topic']:
            return set([
                'pull-request/%s' % msg['msg']['pullrequest']['id'],
                'project/%s' % project,
            ])

        return set([])



import fedmsg.meta.base
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class ByUpdate(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return 'bodhi.update.comment' in msg['topic']

    def matches(self, a, b, **config):
        """ The comments must all be on the same bodhi update """
        a, b = a['msg'], b['msg']
        if a['comment']['update_title'] != b['comment']['update_title']:
            return False
        return True

    def merge(self, constituents, **config):
        N = len(constituents)
        msg = constituents[0]['msg']

        tmpl = self.produce_template(constituents, **config)
        agents = self.list_to_series(list(tmpl['usernames']))
        update = msg['comment']['update_title']
        subtitle = '{agents} commented on {update}'
        tmpl['subtitle'] = subtitle.format(agents=agents, update=update)
        tmpl['secondary_icon'] = tmpl['icon']
        base = 'https://admin.fedoraproject.org/updates/%s/'
        tmpl['link'] = base % update
        return tmpl


class ByUser(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return 'bodhi.update.comment' in msg['topic']

    def matches(self, a, b, **config):
        """ The comments must all be by the same user """
        if a['msg']['agent'] != b['msg']['agent']:
            return False
        return True

    def merge(self, constituents, **config):
        tmpl = self.produce_template(constituents, **config)
        msg = constituents[0]
        agent = msg['msg']['agent']
        updates = self.list_to_series([
            msg['msg']['comment']['update_title'] for msg in constituents])
        subtitle = '{agent} commented on {updates}'
        tmpl['subtitle'] = subtitle.format(agent=agent, updates=updates)
        tmpl['secondary_icon'] = avatar_url(agent)
        base = 'https://admin.fedoraproject.org/updates/user/%s/'
        tmpl['link'] = base % agent
        return tmpl



import collections
import threading
import socket
from hashlib import sha256, md5

_fas_cache = {}
_fas_cache_lock = threading.Lock()

import logging
log = logging.getLogger("moksha.hub")

try:
    from six.moves.urllib import parse
except ImportError:
    # Really really old 'six' doesn't have this move.. so we fall back to
    # python-2 only usage.  If we're on an old 'six', then we can assume that
    # we must also be on an old Python.
    import urllib as parse


def _ordered_query_params(params):
    # if OrderedDict is available, preserver order of params
    #  to make this easily testable on PY3
    if hasattr(collections, 'OrderedDict'):
        retval = collections.OrderedDict(params)
    else:
        retval = dict(params)
    return retval

def avatar_url(username, size=64, default='retro'):
    openid = "http://%s.id.fedoraproject.org/" % username
    return avatar_url_from_openid(openid, size, default)


def avatar_url_from_openid(openid, size=64, default='retro', dns=False):
    """
    Our own implementation since fas doesn't support this nicely yet.
    """

    if dns:
        # This makes an extra DNS SRV query, which can slow down our webapps.
        # It is necessary for libravatar federation, though.
        import libravatar
        return libravatar.libravatar_url(
            openid=openid,
            size=size,
            default=default,
        )
    else:
        params = _ordered_query_params([('s', size), ('d', default)])
        query = parse.urlencode(params)
        hash = sha256(openid.encode('utf-8')).hexdigest()
        return "https://seccdn.libravatar.org/avatar/%s?%s" % (hash, query)


def avatar_url_from_email(email, size=64, default='retro', dns=False):
    """
    Our own implementation since fas doesn't support this nicely yet.
    """

    if dns:
        # This makes an extra DNS SRV query, which can slow down our webapps.
        # It is necessary for libravatar federation, though.
        import libravatar
        return libravatar.libravatar_url(
            email=email,
            size=size,
            default=default,
        )
    else:
        params = _ordered_query_params([('s', size), ('d', default)])
        query = parse.urlencode(params)
        hash = md5(email.encode('utf-8')).hexdigest()
        return "https://seccdn.libravatar.org/avatar/%s?%s" % (hash, query)


def make_fas_cache(**config):
    global _fas_cache
    if _fas_cache:
        return _fas_cache

    log.warn("No previous fas cache found.  Looking to rebuild.")

    try:
        import fedora.client
        import fedora.client.fas2
    except ImportError:
        log.warn("No python-fedora installed.  Not caching fas.")
        return {}

    if not 'fas_credentials' in config:
        log.warn("No fas_credentials found.  Not caching fas.")
        return {}

    creds = config['fas_credentials']

    default_url = 'https://admin.fedoraproject.org/accounts/'
    fasclient = fedora.client.fas2.AccountSystem(
        base_url=creds.get('base_url', default_url),
        username=creds['username'],
        password=creds['password'],
    )

    timeout = socket.getdefaulttimeout()
    socket.setdefaulttimeout(600)
    try:
        log.info("Downloading FAS cache")
        request = fasclient.send_request('/user/list',
                                         req_params={'search': '*'},
                                         auth=True)
    except fedora.client.ServerError as e:
        log.warning("Failed to download fas cache %r" % e)
        return {}
    finally:
        socket.setdefaulttimeout(timeout)

    log.info("Caching necessary user data")
    for user in request['people']:
        nick = user['ircnick']
        if nick:
            _fas_cache[nick] = user['username']

        email = user['email']
        if email:
            _fas_cache[email] = user['username']

    del request
    del fasclient
    del fedora.client.fas2

    return _fas_cache


def nick2fas(nickname, **config):
    log.debug("Acquiring _fas_cache_lock for nicknames.")
    with _fas_cache_lock:
        log.debug("Got _fas_cache_lock for nicknames.")
        fas_cache = make_fas_cache(**config)
        result = fas_cache.get(nickname, nickname)
    log.debug("Released _fas_cache_lock for nicknames.")
    return result


def email2fas(email, **config):
    if email.endswith('@fedoraproject.org'):
        return email.rsplit('@', 1)[0]

    log.debug("Acquiring _fas_cache_lock for emails.")
    with _fas_cache_lock:
        log.debug("Got _fas_cache_lock for emails.")
        fas_cache = make_fas_cache(**config)
        result = fas_cache.get(email, email)
    log.debug("Released _fas_cache_lock for emails.")
    return result



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor

from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url

import datetime
from pytz import UTC

try:
    import koji
except ImportError:
    koji = None


_build_template = """Package:    {name}-{version}-{release}
Status:     {status}
Built by:   {owner_name}
ID:         {id}
Started:    {started}
Finished:   {finished}

"""

_task_header_template = """Task {id} on {build_host}
Task Type: {method} ({arch})
Link: {url}
"""

import logging
log = logging.getLogger('fedmsg.meta.buildsys')


class KojiProcessor(BaseProcessor):
    __name__ = "buildsys"
    __description__ = "the Fedora build system"
    __link__ = "http://koji.fedoraproject.org/koji"
    __docs__ = "https://fedoraproject.org/wiki/Using_the_Koji_build_system"
    __obj__ = "Koji Builds"
    __icon__ = ("https://fedoraproject.org/w/uploads/2/20/"
                "Artwork_DesignService_koji-icon-48.png")

    @classmethod
    def _fill_task_template(cls, sess, taskid):
        file_base = 'https://kojipkgs.fedoraproject.org/work/'

        info = sess.getTaskInfo(taskid)
        if info['host_id'] is None:
            info['build_host'] = '(unscheduled)'
        else:
            host = sess.getHost(info['host_id'])
            info['build_host'] = host['name']

        weburl = sess.baseurl.rsplit('/', 1)[0] + '/koji/'
        info['url'] = weburl + 'taskinfo?taskID=%i' % info['id']

        retval = _task_header_template.format(**info)

        result = None
        try:
            result = sess.getTaskResult(taskid)
        except Exception as e:
            log.warning(unicode(e))
            retval += "\n" + unicode(e) + "\n"

        if result:
            for kind in ['logs', 'rpms', 'srpms']:
                if kind in result:
                    retval += kind + ":\n"
                    for item in result[kind]:
                        retval += "  " + file_base + item + "\n"

            for kind in ['srpm']:
                if kind in result:
                    retval += kind + ":\n  " + file_base + result[kind] + "\n"

        children = sess.getTaskChildren(taskid)
        for child in sorted(children, key=lambda d: d['completion_ts']):
            retval += "\n" + cls._fill_task_template(sess, child['id'])

        return retval

    @classmethod
    def _fill_build_template(cls, sess, build):
        full_build = sess.getBuild(build['build_id'])
        lookup = dict(zip(*zip(*koji.BUILD_STATES.items())[::-1]))
        full_build['status'] = lookup[full_build['state']].lower()

        fmt = '%a, %d %b %Y %H:%M:%S %Z'

        try:
            dt = datetime.datetime.fromtimestamp(
                full_build['creation_ts'], UTC).strftime(fmt)
        except TypeError:
            dt = ''
        full_build['started'] = dt

        try:
            dt = datetime.datetime.fromtimestamp(
                full_build['completion_ts'], UTC).strftime(fmt)
        except TypeError:
            dt = ''
        full_build['finished'] = dt

        try:
            _build_str = _build_template.format(**full_build)
        except Exception as e:
            log.warning(unicode(e))
            _build_str = unicode(e) + "\n"

        task_id = full_build['task_id']
        if task_id is None:
            _task_str = "Build imported into koji\n"
        else:
            try:
                _task_str = "Closed tasks:\n-------------\n"
                _task_str += cls._fill_task_template(sess, task_id)
            except Exception as e:
                log.warning(unicode(e))
                _task_str = unicode(e) + "\n"

        return _build_str + _task_str

    def long_form(self, msg, **config):
        instance = msg['msg'].get('instance', 'primary')

        if instance == 'primary':
            url = "https://koji.fedoraproject.org/kojihub"
        elif instance == 'ppc':
            url = "http://ppc.koji.fedoraproject.org/kojihub"
        elif instance == 's390':
            url = "http://s390.koji.fedoraproject.org/kojihub"
        elif instance == 'arm':
            url = "http://arm.koji.fedoraproject.org/kojihub"

        if 'buildsys.build.state.change' in msg['topic'] and koji:
            session = koji.ClientSession(url)
            build = msg['msg']
            long_form = self._fill_build_template(session, build)
            return long_form
        if 'buildsys.task.state.change' in msg['topic'] and koji:
            session = koji.ClientSession(url)
            taskid = msg['msg']['id']
            try:
                long_form = self._fill_task_template(session, taskid)
            except Exception as e:
                log.warning(unicode(e))
                long_form = unicode(e)
            return long_form

    def subtitle(self, msg, **config):
        inst = msg['msg'].get('instance', 'primary')
        if inst == 'primary':
            inst = ''
        else:
            inst = ' (%s)' % inst

        if 'buildsys.tag' in msg['topic']:
            tmpl = self._(
                "{owner}'s {name}-{version}-{release} tagged "
                "into {tag} by {user}{inst}"
            )
            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.untag' in msg['topic']:
            tmpl = self._(
                "{owner}'s {name}-{version}-{release} untagged "
                "from {tag} by {user}{inst}"
            )
            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.repo.init' in msg['topic']:
            tmpl = self._('Repo initialized:  {tag}{inst}')
            return tmpl.format(inst=inst, tag=msg['msg'].get('tag', 'unknown'))
        elif 'buildsys.repo.done' in msg['topic']:
            tmpl = self._('Repo done:  {tag}{inst}')
            return tmpl.format(inst=inst, tag=msg['msg'].get('tag', 'unknown'))
        elif 'buildsys.package.list.change' in msg['topic']:
            tmpl = self._(
                "Package list change for {package}:  '{tag}'{inst}")
            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.rpm.sign' in msg['topic']:
            tmpl = self._('Koji build '
                          '{name}-{version}-{release}.{arch}.rpm '
                          'signed with sigkey \'{sigkey}\'')
            return tmpl.format(**msg['msg']['info'])
        elif 'buildsys.build.state.change' in msg['topic']:
            templates = [
                self._("{owner}'s {name}-{version}-{release} "
                       "started building{inst}"),
                self._("{owner}'s {name}-{version}-{release} "
                       "completed{inst}"),
                self._("{owner}'s {name}-{version}-{release} "
                       "was deleted{inst}"),
                self._("{owner}'s {name}-{version}-{release} "
                       "failed to build{inst}"),
                self._("{owner}'s {name}-{version}-{release} "
                       "was cancelled{inst}"),
            ]
            tmpl = templates[msg['msg']['new']]

            # If there was no owner of the build, chop off the prefix.
            if not msg['msg']['owner']:
                tmpl = tmpl[len("{owner}'s "):]

            return tmpl.format(inst=inst, **msg['msg'])
        elif 'buildsys.task.state.change' in msg['topic']:
            templates = {
                'OPEN': self._(
                    "{owner}'s scratch build of {srpm} started{inst}"),
                'FAILED': self._(
                    "{owner}'s scratch build of {srpm} failed{inst}"),
                'CLOSED': self._(
                    "{owner}'s scratch build of {srpm} completed{inst}"),
                'CANCELED': self._(
                    "{owner}'s scratch build of {srpm} "
                    "was cancelled{inst}"),
            }
            default = self._(
                "{owner}'s scratch build of {srpm} changed{inst}")
            tmpl = templates.get(msg['msg']['new'], default)

            # If there was no owner of the build, chop off the prefix.
            if not msg['msg']['owner']:
                tmpl = tmpl[len("{owner}'s "):]

            return tmpl.format(inst=inst, **msg['msg'])

    def secondary_icon(self, msg, **config):
        owner = msg['msg'].get('owner')

        if owner:
            return avatar_url(owner)

        return self.__icon__

    def usernames(self, msg, **config):
        if 'buildsys.tag' in msg['topic']:
            return set([
                msg['msg']['owner'],
                msg['msg']['user'],
            ])
        elif 'buildsys.untag' in msg['topic']:
            return set([
                msg['msg']['owner'],
                msg['msg']['user'],
            ])
        elif 'buildsys.repo.init' in msg['topic']:
            return set()
        elif 'buildsys.repo.done' in msg['topic']:
            return set()
        elif 'buildsys.package.list.change' in msg['topic']:
            return set()
        elif 'buildsys.rpm.sign' in msg['topic']:
            return set()
        elif 'buildsys.build.state.change' in msg['topic']:
            if msg['msg']['owner']:
                return set([
                    msg['msg']['owner'],
                ])

            # Sometimes there is no owner
            return set()
        elif 'buildsys.task.state.change' in msg['topic']:
            if msg['msg']['owner']:
                return set([
                    msg['msg']['owner'],
                ])

            # Sometimes there is no owner
            return set()

    def packages(self, msg, **config):
        if 'buildsys.tag' in msg['topic']:
            return set([msg['msg']['name']])
        elif 'buildsys.untag' in msg['topic']:
            return set([msg['msg']['name']])
        elif 'buildsys.repo.init' in msg['topic']:
            return set([])
        elif 'buildsys.repo.done' in msg['topic']:
            return set([])
        elif 'buildsys.package.list.change' in msg['topic']:
            return set([msg['msg']['package']])
        elif 'buildsys.rpm.sign' in msg['topic']:
            return set([msg['msg']['info']['name']])
        elif 'buildsys.build.state.change' in msg['topic']:
            return set([msg['msg']['name']])
        elif 'buildsys.task.state.change' in msg['topic']:
            # We can't *really* associate scratch builds with a package,
            # honestly.
            return set([])

        return set()

    def link(self, msg, **config):

        instance = msg['msg'].get('instance', 'primary')
        if instance == 'primary':
            base = "http://koji.fedoraproject.org/koji"
        elif instance == 'ppc':
            base = "http://ppc.koji.fedoraproject.org/koji"
        elif instance == 's390':
            base = "http://s390.koji.fedoraproject.org/koji"
        elif instance == 'arm':
            base = "http://arm.koji.fedoraproject.org/koji"
        else:
            raise NotImplementedError("Unhandled instance")

        # One last little switch-a-roo for stg
        if '.stg.' in msg['topic']:
            base = "http://koji.stg.fedoraproject.org/koji"

        if 'buildsys.tag' in msg['topic'] and 'tag_id' in msg['msg']:
            return base + "/taginfo?tagID=%i" % (
                msg['msg']['tag_id'])
        elif 'buildsys.untag' in msg['topic'] and 'tag_id' in msg['msg']:
            return base + "/taginfo?tagID=%i" % (
                msg['msg']['tag_id'])
        elif 'buildsys.repo.init' in msg['topic'] and 'tag_id' in msg['msg']:
            return base + "/taginfo?tagID=%i" % (
                msg['msg']['tag_id'])
        elif 'buildsys.repo.done' in msg['topic'] and 'tag_id' in msg['msg']:
            return base + "/taginfo?tagID=%i" % (
                msg['msg']['tag_id'])
        elif 'buildsys.build.state.change' in msg['topic']:
            return base + "/buildinfo?buildID=%i" \
                % (msg['msg']['build_id'])
        elif 'buildsys.task.state.change' in msg['topic']:
            return base + "/taskinfo?taskID=%i" \
                % (msg['msg']['id'])
        elif 'buildsys.package.list.change' in msg['topic']:
            return None
        elif 'buildsys.rpm.sign' in msg['topic']:
            return base + "/buildinfo?buildID=%i" \
                % (msg['msg']['info']['build_id'])
        else:
            return base

    def objects(self, msg, **config):
        instance = msg['msg'].get('instance', 'primary')

        if 'buildsys.tag' in msg['topic']:
            return set([
                '/'.join([
                    instance,
                    'builds',
                    msg['msg']['name'],
                    msg['msg']['version'],
                    msg['msg']['release'],
                ]),
                '/'.join([
                    instance,
                    'tags',
                    msg['msg']['tag'],
                ]),
            ])
        elif 'buildsys.untag' in msg['topic']:
            return set([
                '/'.join([
                    instance,
                    'builds',
                    msg['msg']['name'],
                    msg['msg']['version'],
                    msg['msg']['release'],
                ]),
                '/'.join([
                    instance,
                    'tags',
                    msg['msg'].get('tag', 'unknown'),
                ]),
            ])
        elif 'buildsys.build.state.change' in msg['topic']:
            return set(['/'.join([
                instance,
                'builds',
                msg['msg']['name'],
                msg['msg']['version'],
                msg['msg']['release'],
            ])])
        elif 'buildsys.task.state.change' in msg['topic']:
            return set(['/'.join([
                instance,
                'scratch_builds',
                msg['msg']['srpm'],
            ])])
        elif 'buildsys.repo.init' in msg['topic']:
            return set(['/'.join([
                instance,
                'repos',
                msg['msg'].get('tag', 'unknown'),
            ])])
        elif 'buildsys.repo.done' in msg['topic']:
            return set(['/'.join([
                instance,
                'repos',
                msg['msg'].get('tag', 'unknown'),
            ])])
        elif 'buildsys.package.list.change' in msg['topic']:
            return set(['/'.join([
                instance,
                'tags',
                msg['msg'].get('tag', 'unknown'),
            ])])
        elif 'buildsys.rpm.sign' in msg['topic']:
            return set(['signatures/' + msg['msg']['info']['name']])

        return set()



# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for fas messages. """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from fedmsg_meta_fedora_infrastructure.tests.common import add_doc


class TestFASUserCreateLegacy(Base):
    """ Support old fas messages (the new ones look different).

    :mod:`fedmsg.meta` needs to be able to handle these since they are stored
    *forever* in datanommer.
    """
    expected_title = "fas.user.create"
    expected_subti = "New FAS account:  'ralph'  (created by 'ralph')"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph'])
    msg = {
        u'i': 1,
        u'timestamp': 1344432054.8098609,
        u'topic': u'org.fedoraproject.stg.fas.user.create',
        u'msg': {
            u'user': {
                u'username': u'ralph'
            },
            u'agent': {
                u'username': u'ralph'
            }
        }
    }


class TestFASUserCreate(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a new user account is created.
    """
    expected_title = "fas.user.create"
    expected_subti = "New FAS account:  'ralph'  (created by 'ralph')"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph'])
    msg = {
        u'i': 1,
        u'timestamp': 1344432054.8098609,
        u'topic': u'org.fedoraproject.stg.fas.user.create',
        u'msg': {
            u'user': u'ralph',
            u'agent': u'ralph',
        }
    }


class TestFASEditProfile(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user's account is modified.
    Information about which account, what fields changed, and who did the
    changing are included in the message body.  For example:
    """
    expected_title = "fas.user.update"
    expected_subti = "ralph edited the following fields of ralph's " + \
        "FAS profile:  comments"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.user.update',
        u'msg': {
            u'fields': [u'comments'],
            u'user': u'ralph',
            u'agent': u'ralph',
        }
    }


class TestFASEditGroupLegacy(Base):
    """ Support old fas messages (the new ones look different).

    :mod:`fedmsg.meta` needs to be able to handle these since they are stored
    *forever* in datanommer.
    """
    expected_title = "fas.group.update"
    expected_subti = "ralph edited the following fields of the " + \
        "ambassadors FAS group:  display_name"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.update',
        u'msg': {
            u'fields': [u'display_name'],
            u'group': {u'name': u'ambassadors'},
            u'agent': {u'username': u'ralph'},
        }
    }


class TestFASEditGroup(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a group's properties are
    modified.  For example:
    """
    expected_title = "fas.group.update"
    expected_subti = "ralph edited the following fields of the " + \
        "ambassadors FAS group:  display_name"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.update',
        u'msg': {
            u'fields': [u'display_name'],
            u'group': u'ambassadors',
            u'agent': u'ralph',
        }
    }


class TestFASGroupCreate(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a new group is created.
    """
    expected_title = "fas.group.create"
    expected_subti = "ralph created new FAS group ambassadors"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.create',
        u'msg': {
            u'group': u'ambassadors',
            u'agent': u'ralph',
        }
    }


class TestFASGroupCreateLegacy(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a new group is created.
    """
    expected_title = "fas.group.create"
    expected_subti = "ralph created new FAS group ambassadors"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.create',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'agent': {u'username': u'ralph'},
        }
    }


class TestFASRoleUpdate(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user's role in a particular
    group changes.
    """
    expected_title = "fas.role.update"
    expected_subti = "toshio changed ralph's role in the ambassadors group to sponsor"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "085a38b9cf600926924645b292f0b7121a98a5165e559e0ad1882cfe33c6b395" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.role.update',
        u'msg': {
            u'group': u'ambassadors',
            u'user': u'ralph',
            u'agent': u'toshio',
            u'status': u'sponsor',
        }
    }


class TestLegacyFASRoleUpdate(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user's role in a particular
    group changes.
    This is a test whether or not we can still handle messages without status.
    """
    expected_title = "fas.role.update"
    expected_subti = "toshio changed ralph's role in the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "085a38b9cf600926924645b292f0b7121a98a5165e559e0ad1882cfe33c6b395" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.role.update',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'toshio'},
        }
    }


class TestLegacy2FASGroupRemove(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user is **removed** from a
    particular group.
    """
    expected_title = "fas.group.member.remove"
    expected_subti = "toshio removed ralph from the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "085a38b9cf600926924645b292f0b7121a98a5165e559e0ad1882cfe33c6b395" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.remove',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'toshio'},
        }
    }


class TestLegacy2FASGroupSponsor(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user's request to join a
    restricted group is **sponsored** by an authorized user.
    """
    expected_title = "fas.group.member.sponsor"
    expected_subti = "toshio sponsored ralph's membership " + \
        "in the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "085a38b9cf600926924645b292f0b7121a98a5165e559e0ad1882cfe33c6b395" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.sponsor',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'toshio'},
        }
    }


class TestLegacy2FASGroupApply(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user **requests to join** a
    particular group.
    """
    expected_title = "fas.group.member.apply"
    expected_subti = "ralph applied for ralph's membership " + \
        "in the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.apply',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'ralph'},
        }
    }

class TestFASGroupRemove(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user is **removed** from a
    particular group.
    """
    expected_title = "fas.group.member.remove"
    expected_subti = "toshio removed ralph from the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "085a38b9cf600926924645b292f0b7121a98a5165e559e0ad1882cfe33c6b395" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.remove',
        u'msg': {
            u'group': u'ambassadors',
            u'user':  u'ralph',
            u'agent': u'toshio',
        }
    }


class TestFASGroupSponsor(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user's request to join a
    restricted group is **sponsored** by an authorized user.
    """
    expected_title = "fas.group.member.sponsor"
    expected_subti = "toshio sponsored ralph's membership " + \
        "in the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "085a38b9cf600926924645b292f0b7121a98a5165e559e0ad1882cfe33c6b395" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.sponsor',
        u'msg': {
            u'group': u'ambassadors',
            u'user': u'ralph',
            u'agent': u'toshio',
        }
    }


class TestFASGroupApply(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user **requests to join** a
    particular group.
    """
    expected_title = "fas.group.member.apply"
    expected_subti = "ralph applied for ralph's membership " + \
        "in the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.apply',
        u'msg': {
            u'group': u'ambassadors',
            u'user': u'ralph',
            u'agent': u'ralph',
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



""" Code to generate doc/topics.rst during 'sphinx-build'.

This code:

- Uses :mod:`nose` to find all the fedmsg.meta unittests.
- Extracts all the metadata and docstrings from those tests.
- Uses all that to generate a giant .rst document of all the fedmsg
  topics and what they are about with example messages.

"""

import os
import nose
import pprint
import textwrap

header = """
List of Message Topics
======================

.. DO NOT EDIT THIS DOCUMENT.

.. It is autogenerated from fedmsg_meta_fedora_infrastructure/doc_utilities.py

This document lists all the topics coming out the Fedora
Infrastructure fedmsg bus.  Example messages are included
as well as descriptions and sample output from ``fedmsg.meta``.

.. note:: All topics from Fedora Infrastructure are prefixed with
   ``org.fedoraproject.prod.``, but the :term:`topic_prefix` is omitted here
   for brevity.  For instance, the item listed as ``git.branch`` will
   actually be broadcast as ``org.fedoraproject.prod.git.branch``.

.. note:: Message bodies can contain some useful information, but be wary.
   We have done as good a job as we can *securing* fedmsg, but it is still
   a new system.  If you receive a message from pkgdb claiming that "ralph"
   is the new owner of the kernel, you should still *check* with the *actual*
   pkgdb service that this is the case.  Write code against fedmsg messages
   as a tip, but always check the authoritative source before taking any
   programmatic action.

"""

metadata_template = """
The example message above, when passed to various routines in the
:mod:`fedmsg.meta` module, will produce the following outputs:

- :func:`fedmsg.meta.msg2title`

  - {title}

- :func:`fedmsg.meta.msg2subtitle`

  - {subtitle}

- :func:`fedmsg.meta.msg2link`

  - {link}

- :func:`fedmsg.meta.msg2icon`

  - {icon}

.. image:: {icon}
   :height: 64px
   :width: 64px

- :func:`fedmsg.meta.msg2secondary_icon`

  - {secondary_icon}

.. image:: {secondary_icon}
   :height: 64px
   :width: 64px

- :func:`fedmsg.meta.msg2usernames`

 - ``{usernames}``

- :func:`fedmsg.meta.msg2packages`

 - ``{packages}``

- :func:`fedmsg.meta.msg2objects`

 - ``{objects}``

"""

outfile = None


def write(fname, s=''):
    global outfile
    if not outfile:
        outfile = open(fname, 'w')

    outfile.write(s + '\n')


def load_classes(module):
    return list(nose.loader.defaultTestLoader().loadTestsFromModule(module))


def make_topics_doc(output_dir):

    fname = output_dir + "/topics.rst"

    global outfile
    import fedmsg_meta_fedora_infrastructure.tests as source_module
    test_classes = load_classes(source_module)

    # TODO -- get the logger and announce messages
    #import fedmsg
    #filename = fedmsg.__file__
    #folder = os.path.sep + os.path.join(*filename.split('/')[:-1])
    #test_classes = load_classes(folder)

    # Strip out the conglomerator tests which are more complicated.
    test_classes = [cls for cls in test_classes if hasattr(cls.context, 'msg')]

    write(fname, header)

    for cls in test_classes:
        if cls.context.msg:
            # Adjust {stg,dev} to prod.
            cls.context.msg['topic'] = cls.context.msg['topic']\
                .replace('.stg.', '.prod.')\
                .replace('.dev.', '.prod.')
            cls.context.expected_title = cls.context.expected_title\
                .replace('.stg.', '.prod.')\
                .replace('.dev.', '.prod.')

            topic = '.'.join(cls.context.msg['topic'].split('.')[3:])
            cls.__topic = topic
        else:
            cls.__topic = None

    comparator = lambda a, b: cmp(a.__topic, b.__topic)
    test_classes = sorted(test_classes, comparator)

    seen = []
    for cls in test_classes:
        if cls.context.msg:
            topic = cls.__topic

            # Ignore tests that check old messages.
            if 'Legacy' in cls.context.__name__:
                continue

            modname = topic.split('.')[0]
            if not modname in seen:
                seen.append(modname)
                write(fname, modname)
                write(fname, "-" * len(modname))
                write(fname)

            write(fname, topic)
            write(fname, "~" * len(topic))
            write(fname)

            # I would use __doc__ here, but something that nose is doing is
            # stripping the __doc__ from my original unit tests.  Instead,
            # we'll use our own 'doc' attribute which is a little clumsy.
            if getattr(cls.context, 'doc', None):
                write(fname, textwrap.dedent("    " + cls.context.doc.strip()))
                write(fname)

            write(fname, ".. code-block:: python")
            write(fname, '\n    ' + pprint.pformat(cls.context.msg, indent=2)
                  .replace('\n', '\n    '))
            write(fname)
            write(fname, metadata_template.format(
                link=cls.context.expected_link,
                title=cls.context.expected_title,
                subtitle=cls.context.expected_subti,
                usernames=cls.context.expected_usernames,
                packages=cls.context.expected_packages,
                objects=cls.context.expected_objects,
                icon=cls.context.expected_icon,
                secondary_icon=cls.context.expected_secondary_icon,
            ))
            write(fname)

    outfile.close()

if __name__ == '__main__':
    make_topics_doc()



# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
""" Tests for messages from FMN """

import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestFilterUpdateRules(Base):
    """ Messages of this type are published whenever someone **updates
    one of their notification rules** in the `Fedora Notifications
    <https://apps.fedoraproject.org/notifications>`_ app.
    """
    expected_title = "fmn.filter.update"
    expected_subti = "ralph updated the rules on a fmn irc filter"
    expected_link = "https://apps.fedoraproject.org/notifications/"
    expected_icon = "https://apps.fedoraproject.org/img/icons/fedmsg.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['ralph/irc/filter/rules'])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.prod.fmn.filter.update",
        "msg": {
            'openid': 'ralph.id.fedoraproject.org',
            'context': 'irc',
            'changed': 'rules',
        }
    }


class TestPreferenceUpdateBatchValues(Base):
    """ Messages of this type are published whenever someone **updates
    their batch values** in the `Fedora Notifications
    <https://apps.fedoraproject.org/notifications>`_ app.

    Batch values are the parameters used to determine how to send notification
    digests like "how many messages should we accumulate before forwarding them
    to you" or "how many days should we wait before forwarding you a digest".
    """
    expected_title = "fmn.preference.update"
    expected_subti = "ralph updated their email digest options"
    expected_link = "https://apps.fedoraproject.org/notifications/"
    expected_icon = "https://apps.fedoraproject.org/img/icons/fedmsg.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['ralph/email/preference/batch_values'])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.prod.fmn.preference.update",
        "msg": {
            'openid': 'ralph.id.fedoraproject.org',
            'context': 'email',
            'changed': 'batch_values',
        }
    }


class TestPreferenceUpdateDetails(Base):
    """ Messages of this type are published whenever someone **updates
    their delivery details** in the `Fedora Notifications
    <https://apps.fedoraproject.org/notifications>`_ app.

    Delivery details are the values used to figure out how to deliver messages,
    like "email address" or "irc nick".
    """
    expected_title = "fmn.preference.update"
    expected_subti = "ralph updated their email delivery details"
    expected_link = "https://apps.fedoraproject.org/notifications/"
    expected_icon = "https://apps.fedoraproject.org/img/icons/fedmsg.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['ralph/email/preference/details'])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.prod.fmn.preference.update",
        "msg": {
            'openid': 'ralph.id.fedoraproject.org',
            'context': 'email',
            'changed': 'details',
        }
    }


class TestPreferenceUpdateEnabled(Base):
    """ Messages of this type are published whenever someone **toggles
    delivery of messages** in the `Fedora Notifications
    <https://apps.fedoraproject.org/notifications>`_ app.
    """
    expected_title = "fmn.preference.update"
    expected_subti = "ralph toggled their flow of android messages"
    expected_link = "https://apps.fedoraproject.org/notifications/"
    expected_icon = "https://apps.fedoraproject.org/img/icons/fedmsg.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['ralph/android/preference/enabled'])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.prod.fmn.preference.update",
        "msg": {
            'openid': 'ralph.id.fedoraproject.org',
            'context': 'android',
            'changed': 'enabled',
        }
    }


class TestPreferenceUpdateFilters(Base):
    """ Messages of this type are published whenever someone **toggles
    delivery of messages** in the `Fedora Notifications
    <https://apps.fedoraproject.org/notifications>`_ app.
    """
    expected_title = "fmn.preference.update"
    expected_subti = "ralph updated their irc filters"
    expected_link = "https://apps.fedoraproject.org/notifications/"
    expected_icon = "https://apps.fedoraproject.org/img/icons/fedmsg.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['ralph/irc/preference/filters'])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.prod.fmn.preference.update",
        "msg": {
            'openid': 'ralph.id.fedoraproject.org',
            'context': 'irc',
            'changed': 'filters',
        }
    }


class TestConfirmationUpdateValue(Base):
    """ Messages of this type are published whenever **a confirmation value
    changes** in the `Fedora Notifications
    <https://apps.fedoraproject.org/notifications>`_ app.
    """
    expected_title = "fmn.confirmation.update"
    expected_subti = "the value of one of ralph's pending " + \
        "confirmations changed"
    expected_link = "https://apps.fedoraproject.org/notifications/"
    expected_icon = "https://apps.fedoraproject.org/img/icons/fedmsg.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['ralph/irc/confirmation/value'])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.prod.fmn.confirmation.update",
        "msg": {
            'openid': 'ralph.id.fedoraproject.org',
            'context': 'irc',
            'changed': 'value',
        }
    }


class TestConfirmationUpdateStatus(Base):
    """ Messages of this type are published whenever **the status of a
    confirmation changes** in the `Fedora Notifications
    <https://apps.fedoraproject.org/notifications>`_ app.
    """
    expected_title = "fmn.confirmation.update"
    expected_subti = "the status of one of ralph's pending " + \
        "confirmations changed"
    expected_link = "https://apps.fedoraproject.org/notifications/"
    expected_icon = "https://apps.fedoraproject.org/img/icons/fedmsg.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['ralph/irc/confirmation/status'])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.prod.fmn.confirmation.update",
        "msg": {
            'openid': 'ralph.id.fedoraproject.org',
            'context': 'irc',
            'changed': 'status',
        }
    }

class TestRuleUpdate(Base):
    """ Messages of this type are published whenever **a rule
    is updated** in the `Fedora Notifications
    <https://apps.fedoraproject.org/notifications>`_ app.
    """
    expected_title = "fmn.rule.update"
    expected_subti = "raveit65 updated the filters on a fmn email rule"
    expected_link = "https://apps.fedoraproject.org/notifications/"
    expected_icon = "https://apps.fedoraproject.org/img/icons/fedmsg.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "15321e38b8a94a429e68a68a0f5029371b987c3aa7a0072326538b65ae54ec7e"
        "?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['raveit65'])
    expected_objects = set(['raveit65/email/rule/filters'])
    msg = {
        "i": 1, 
        "timestamp": 1427936259.0, 
        "msg_id": "2015-ca011930-b7ba-4355-a75d-079ffbe88fd2", 
        "topic": "org.fedoraproject.prod.fmn.rule.update", 
        "source_version": "0.6.5", 
        "msg": {
            "openid": "raveit65.id.fedoraproject.org", 
            "changed": "filters", 
            "context": "email"
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for mirrormanager2 messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestCrawlerStart(Base):
    """ `Mirrormanager <https://mirrors.fedoraproject.org>`_ is the system that
    (surprise) manages the mirror network for Fedora (and others).  To
    accomplish this task, it does lots of things.

    One of those things is to regularly crawl the list of mirrors and determine
    who has up-to-date content and who has stale content. *This* message is one
    that gets published by its backend crawler when it **starts** its task.
    """

    expected_title = "mirrormanager.crawler.start"
    expected_subti = "mirrormanager's crawler started a crawl of 1 mirrors"
    expected_link = "https://mirrors.fedoraproject.org"
    expected_icon = "https://apps.fedoraproject.org/img/icons/downloads.png"
    expected_secondary_icon = expected_icon
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['mirrors/mirror.fpt-telecom.fpt.net'])

    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1412327834,
        "msg_id": "2014-1f52337a-8dbe-48f1-baad-f2c770c60640",
        "crypto": "x509",
        "topic": "org.fedoraproject.prod.mirrormanager.crawler.start",
        "msg": {
            "hosts": [{
                "comment": "Mirror FPT Telecom",
                "name": "mirror.fpt-telecom.fpt.net",
                "internet2": True,
                "asn_clients": True,
                "country": "VN",
                "admin_active": True,
                "bandwidth_int": 1000,
                "site": {
                    "id": 1043,
                    "name": "mirror.fpt-telecom.fpt.net"
                },
                "private": False,
                "last_crawled": 1374957440.0,
                "internet2_clients": False,
                "id": 1432,
                "user_active": False,
                "last_checked_in": None,
                "last_crawl_duration": 9,
                "asn": None,
                "max_connections": 1
            }]
        }
    }


class TestCrawlerComplete(Base):
    """ `Mirrormanager <https://mirrors.fedoraproject.org>`_ is the system that
    (surprise) manages the mirror network for Fedora (and others).  To
    accomplish this task, it does lots of things.

    One of those things is to regularly crawl the list of mirrors and determine
    who has up-to-date content and who has stale content. *This* message is one
    that gets published by its backend crawler when it **completes** its task.
    """

    expected_title = "mirrormanager.crawler.complete"
    expected_subti = "mirrormanager's crawler finished a crawl of " + \
        "1 mirrors (1 succeeded, 0 failed)"
    expected_link = "https://mirrors.fedoraproject.org"
    expected_icon = "https://apps.fedoraproject.org/img/icons/downloads.png"
    expected_secondary_icon = expected_icon
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['mirrors/mirror.fpt-telecom.fpt.net'])

    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1412327834,
        "msg_id": "2014-1f52337a-8dbe-48f1-baad-f2c770c60640",
        "crypto": "x509",
        "topic": "org.fedoraproject.prod.mirrormanager.crawler.complete",
        "msg": {
            "results": [{
                "rc": 0,
                "host": {
                    "comment": "Mirror FPT Telecom",
                    "name": "mirror.fpt-telecom.fpt.net",
                    "internet2": True,
                    "asn_clients": True,
                    "country": "VN",
                    "admin_active": True,
                    "bandwidth_int": 1000,
                    "site": {
                        "id": 1043,
                        "name": "mirror.fpt-telecom.fpt.net"
                    },
                    "private": False,
                    "last_crawled": 1374957440.0,
                    "internet2_clients": False,
                    "id": 1432,
                    "user_active": False,
                    "last_checked_in": None,
                    "last_crawl_duration": 9,
                    "asn": None,
                    "max_connections": 1
                },
            }]
        }
    }


class TestNetblocksSuccess(Base):
    """ `Mirrormanager <https://mirrors.fedoraproject.org>`_ is the system that
    (surprise) manages the mirror network for Fedora (and others).  To
    accomplish this task, it does lots of things.

    One of those things is to pull in mappings of ASN numbers from publicly
    accessible BGP tables.  It uses this information as part of a larger
    process to try and route clients to mirrors that are close to them.

    *This* message is one that gets published by a backend cronjob when it
    **successfully rebuilds one of its netblocks tables**.
    """

    expected_title = "mirrormanager.netblocks.get"
    expected_subti = "mirrormanager's backend successfully updated its " + \
        "global netblocks file"
    expected_link = "https://mirrors.fedoraproject.org"
    expected_icon = "https://apps.fedoraproject.org/img/icons/downloads.png"
    expected_secondary_icon = expected_icon
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['netblocks/global'])

    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1412327834,
        "msg_id": "2014-1f52337a-8dbe-48f1-baad-f2c770c60640",
        "crypto": "x509",
        "topic": "org.fedoraproject.prod.mirrormanager.netblocks.get",
        "msg": {
            "type": "global",
            "success": True,
        }
    }


class TestNetblocksFailure(Base):
    """ `Mirrormanager <https://mirrors.fedoraproject.org>`_ is the system that
    (surprise) manages the mirror network for Fedora (and others).  To
    accomplish this task, it does lots of things.

    One of those things is to pull in mappings of ASN numbers from publicly
    accessible BGP tables.  It uses this information as part of a larger
    process to try and route clients to mirrors that are close to them.

    *This* message is one that gets published by a backend cronjob when it
    **fails to rebuild one of its netblocks tables**.
    """

    expected_title = "mirrormanager.netblocks.get"
    expected_subti = "mirrormanager's backend failed to update its " + \
        "internet2 netblocks file"
    expected_link = "https://mirrors.fedoraproject.org"
    expected_icon = "https://apps.fedoraproject.org/img/icons/downloads.png"
    expected_secondary_icon = expected_icon
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['netblocks/internet2'])

    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1412327834,
        "msg_id": "2014-1f52337a-8dbe-48f1-baad-f2c770c60640",
        "crypto": "x509",
        "topic": "org.fedoraproject.prod.mirrormanager.netblocks.get",
        "msg": {
            "type": "internet2",
            "success": False,
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()
